<?php

/**
 * @file
 * Content Moderation Node Grants access hooks.
 *
 * Emulating the default Drupal node entity_access behaviours but with
 * content_moderation integration.
 */

use Drupal\node\Entity\NodeType;
use Drupal\node\NodeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflows\Entity\Workflow;

/**
 * Implements hook_node_access_records().
 *
 * @see content_moderation_node_grants_node_grants()
 * @see hook_node_access_records()
 */
function content_moderation_node_grants_node_access_records(NodeInterface $node) {
  $grants = [];

  $owner_id = $node->getOwnerId();

  /* @see \Drupal\node\NodeAccessControlHandler::acquireGrants() */
  /* @see \Drupal\node\NodeGrantDatabaseStorage::writeDefault() */
  /* @see \node_install() */
  // Add the equivalent of the default grant defined by
  // NodeAccessControlHandler::acquireGrants().
  if ($node->isPublished()) {
    $grants[] = [
      'realm' => 'all',
      'gid' => 0,
      'nid' => 0,
      'grant_view' => 1,
      'grant_update' => 0,
      'grant_delete' => 0,
    ];
  }
  else {
    $grants[] = [
      'realm' => 'view any unpublished content',
      'gid' => 0,
      'grant_view' => 1,
      'grant_update' => 0,
      'grant_delete' => 0,
    ];
    if ($owner_id) {
      $grants[] = [
        'realm' => 'view own unpublished content',
        'gid' => $owner_id,
        'grant_view' => 1,
        'grant_update' => 0,
        'grant_delete' => 0,
      ];
    }
  }

  $type_id = $node->bundle();
  $grants[] = [
    'realm' => "edit any $type_id content",
    'gid' => 0,
    'grant_view' => 0,
    'grant_update' => 1,
    'grant_delete' => 0,
  ];
  $grants[] = [
    'realm' => "delete any $type_id content",
    'gid' => 0,
    'grant_view' => 0,
    'grant_update' => 0,
    'grant_delete' => 1,
  ];
  if ($owner_id) {
    $grants[] = [
      'realm' => "edit own $type_id content",
      'gid' => $owner_id,
      'grant_view' => 0,
      'grant_update' => 1,
      'grant_delete' => 0,
    ];
    $grants[] = [
      'realm' => "delete own $type_id content",
      'gid' => $owner_id,
      'grant_view' => 0,
      'grant_update' => 0,
      'grant_delete' => 1,
    ];
  }

  /** @var \Drupal\content_moderation\ModerationInformationInterface $moderation_info */
  $moderation_info = Drupal::service('content_moderation.moderation_information');
  if ($moderation_info->isModeratedEntity($node) && $node->moderation_state) {
    /* @see \Drupal\content_moderation\StateTransitionValidation::getValidTransitions() */
    $workflow = $moderation_info->getWorkflowForEntity($node);
    if ($workflow) {
      $current_state = $node->moderation_state->value ? $workflow->getTypePlugin()->getState($node->moderation_state->value) : $workflow->getTypePlugin()->getInitialState($node);
      foreach ($current_state->getTransitions() as $transition) {
        $grants[] = [
          'realm' => 'use ' . $workflow->id() . ' transition ' . $transition->id(),
          'gid' => 0,
          'grant_view' => 0,
          'grant_update' => 1,
          'grant_delete' => 0,
        ];
      }
    }
  }

  return $grants;
}

/**
 * Implements hook_node_grants().
 *
 * @see content_moderation_entity_access()
 * @see node_query_node_access_alter()
 * @see hook_node_grants()
 */
function content_moderation_node_grants_node_grants(AccountInterface $account, $op) {
  $grants = [];
  $account_id = $account->id();
  if ($op === 'view') {
    if ($account->hasPermission('view any unpublished content')) {
      $grants['view any unpublished content'] = [0];
    }
    elseif ($account_id && $account->hasPermission('view own unpublished content')) {
      $grants['view own unpublished content'] = [$account_id];
    }
  }
  elseif ($op === 'update') {
    /** @var \Drupal\workflows\WorkflowInterface $workflow */
    /* @see \Drupal\content_moderation\Permissions::transitionPermissions() */
    foreach (Workflow::loadMultipleByType('content_moderation') as $workflow) {
      foreach ($workflow->getTypePlugin()->getTransitions() as $transition) {
        $permission = 'use ' . $workflow->id() . ' transition ' . $transition->id();
        if ($account->hasPermission($permission)) {
          $grants[$permission] = [0];
        }
      }
    }
  }

  /* @see \Drupal\node\NodePermissions::nodeTypePermissions() */
  foreach (array_keys(NodeType::loadMultiple()) as $type_id) {
    if ($op === 'update') {
      if ($account->hasPermission("edit any $type_id content")) {
        $grants["edit any $type_id content"] = [0];
      }
      elseif ($account_id && $account->hasPermission("edit own $type_id content")) {
        $grants["edit own $type_id content"] = [$account_id];
      }
    }
    elseif ($op === 'delete') {
      if ($account->hasPermission("delete any $type_id content")) {
        $grants["delete any $type_id content"] = [0];
      }
      elseif ($account_id && $account->hasPermission("delete own $type_id content")) {
        $grants["delete own $type_id content"] = [$account_id];
      }
    }
  }

  return $grants;
}
